import axios, { type AxiosError } from 'axios'
import Router from 'next/router'
import status from 'http-status'
import * as api from '@/api'

axios.interceptors.request.use(function (config) {
  const accessToken = localStorage.getItem('accessToken')
  config.headers.Authorization = `Bearer ${accessToken}`
  return config
})

axios.interceptors.response.use(
  function (response) {
    return response
  },
  async function (error: AxiosError) {
    // fail to refresh token
    if (
      error.config &&
      error.config.url === '/api/auth/token' &&
      error.config.method === 'put'
    ) {
      Router.push('/login')
      return Promise.reject(error)
    }
    switch (error.response?.status) {
      case status.UNAUTHORIZED: {
        const refreshToken = localStorage.getItem('refreshToken')
        if (!refreshToken) {
          Router.push('/login')
          return Promise.reject(error)
        }
        const res = await api.auth.refreshToken({ refreshToken })
        localStorage.setItem('accessToken', res.data.accessToken)
        localStorage.setItem('refreshToken', res.data.refreshToken)
        return error.config && axios.request(error.config)
      }
      default: {
        return Promise.reject(error)
      }
    }
  },
)

export default axios
